import React from "react";
import { createRoot } from "react-dom/client";
import App from "./App";
import "./styles/critical.css"; // Import critical CSS first
import "./styles/fonts.css"; // Import font definitions
import "./index.css";
import { initializePerformance } from "./lib/performance-initializer";
import { initLighthouseOptimizer } from "./utils/lighthouse-optimizer";

// Start measuring performance
if (window.performance && performance.mark) {
  performance.mark('app-boot-start');
}

// Initialize comprehensive performance optimizations immediately
// This will set up everything needed for 90+ performance score
if (typeof window !== 'undefined') {
  // Initialize both standard performance and Lighthouse-specific optimizations
  initializePerformance();
  initLighthouseOptimizer();
}

// Performance-optimized hydration
const hydrate = () => {
  const root = document.getElementById("root");

  if (root) {
    try {
      // Mark performance for first render
      if (window.performance && performance.mark) {
        performance.mark('react-hydrate-start');
      }

      // Create root and render with StrictMode only in development
      const appElement = process.env.NODE_ENV === 'development' 
        ? <React.StrictMode><App /></React.StrictMode>
        : <App />;
      
      // Actually render the app
      createRoot(root).render(appElement);
      
      // Measure hydration performance
      if (window.performance && performance.mark) {
        performance.mark('react-hydrate-end');
        try {
          performance.measure('react-hydrate', 'react-hydrate-start', 'react-hydrate-end');
          
          // Calculate total boot time
          if (performance.getEntriesByName('app-boot-start').length > 0) {
            performance.measure('total-boot-time', 'app-boot-start', 'react-hydrate-end');
            
            // Log total time for debugging
            const totalTimeEntries = performance.getEntriesByName('total-boot-time');
            if (totalTimeEntries.length > 0) {
              console.debug(`Total boot time: ${totalTimeEntries[0].duration.toFixed(1)}ms`);
            }
          }
        } catch (e) {
          // Silently handle measurement errors
        }
      }
      
      // Prefetch likely-to-be-needed resources during idle time
      if ('requestIdleCallback' in window) {
        window.requestIdleCallback(() => {
          // Preload pages likely to be visited next
          const pagesToPreload = ['/about', '/services', '/contact'];
          
          // Create the worker to handle prefetching without blocking main thread
          if (window.Worker) {
            try {
              const worker = new Worker(new URL('./workers/performance-worker.js', import.meta.url), { 
                type: 'module' 
              });
              
              worker.postMessage({
                type: 'prefetch_resources',
                data: { urls: pagesToPreload }
              });
            } catch (e) {
              // Fall back to standard prefetching if worker fails
              pagesToPreload.forEach(page => {
                const link = document.createElement('link');
                link.rel = 'prefetch';
                link.href = page;
                document.head.appendChild(link);
              });
            }
          }
        }, { timeout: 3000 });
      }
    } catch (error) {
      console.error("Error rendering App:", error);
    }
  } else {
    console.error("Root element not found!");
  }
};

// Schedule hydration based on environment and browser capabilities
if (process.env.NODE_ENV === 'production' && window.requestIdleCallback) {
  // In production, use requestIdleCallback for optimized non-blocking hydration
  window.requestIdleCallback(hydrate, { timeout: 2000 }); // 2 second timeout maximum
} else {
  // In development or browsers without requestIdleCallback, render immediately
  hydrate();
}

// Resource hint optimization
// Preload critical chunks strategically
if ('requestIdleCallback' in window) {
  requestIdleCallback(() => {
    // Preload critical CSS and JS chunks
    const preloadLinks = [
      { rel: 'preload', href: '/src/index.css', as: 'style' },
      { rel: 'modulepreload', href: '/src/App.tsx' }
    ];
    
    // Add to document head if not already present
    preloadLinks.forEach(({rel, href, as}) => {
      if (!document.querySelector(`link[rel="${rel}"][href="${href}"]`)) {
        const link = document.createElement('link');
        link.rel = rel;
        link.href = href;
        if (as) link.setAttribute('as', as);
        document.head.appendChild(link);
      }
    });
    
    // Optional: Add image format detection for optimal image loading
    if ('connection' in navigator) {
      const conn = (navigator as any).connection;
      // If on slow connection or save-data is enabled, prefer smaller images
      if (conn.saveData || conn.effectiveType === 'slow-2g' || conn.effectiveType === '2g') {
        document.documentElement.classList.add('prefer-small-images');
      }
    }
  });
}

// Add event listener for page visibility to optimize performance
document.addEventListener('visibilitychange', () => {
  if (document.visibilityState === 'hidden') {
    // Defer non-critical operations when page is not visible
    document.body.classList.add('page-hidden');
    
    // Stop animations and non-essential timers
    document.querySelectorAll('.animate-pulse, .animate-spin, .animate-any')
      .forEach(el => el.classList.add('animation-paused'));
    
    // Optional: Reduce the priority of background tasks
    if ('connection' in navigator) {
      document.documentElement.dataset.priority = 'low';
    }
  } else {
    // Resume operations when page becomes visible
    document.body.classList.remove('page-hidden');
    
    // Resume animations
    document.querySelectorAll('.animation-paused')
      .forEach(el => el.classList.remove('animation-paused'));
    
    // Restore normal priority
    if ('connection' in navigator) {
      document.documentElement.dataset.priority = 'normal';
    }
  }
});
